/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "temphum9.h"

// ------------------------------------------------------------- PRIVATE MACROS 



// -------------------------------------------------------------- PRIVATE TYPES


// ------------------------------------------------------------------ CONSTANTS


// ------------------------------------------------------------------ VARIABLES


// ---------------------------------------------- PRIVATE FUNCTION DECLARATIONS 

static void generic_read_data ( temphum9_t *ctx, uint16_t reg, uint8_t *data_buf, uint8_t len );

static void temphum9_write_data ( temphum9_t *ctx, uint16_t reg, uint8_t *data_buf, uint8_t len );
// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void temphum9_cfg_setup ( temphum9_cfg_t *cfg )
{
    // Communication gpio pins 

    cfg->scl = HAL_PIN_NC;
    cfg->sda = HAL_PIN_NC;
    
    // Additional gpio pins

   //cfg->an   = HAL_PIN_NC;
   //cfg->rst = HAL_PIN_NC;
   //cfg->cs   = HAL_PIN_NC;
   //cfg->pwm = HAL_PIN_NC;
   //cfg->int_pin = HAL_PIN_NC;

    cfg->i2c_speed = I2C_STANDARD_SPEED; 
    cfg->i2c_address = 0x00;
}

TEMPHUM9_RETVAL temphum9_init ( temphum9_t *ctx, temphum9_cfg_t *cfg )
{
     i2c_master_config_t i2c_cfg;

    i2c_master_configure_default( &i2c_cfg );
    i2c_cfg.speed    = cfg->i2c_speed;
    i2c_cfg.pins.scl = cfg->scl;
    i2c_cfg.pins.sda = cfg->sda;

    ctx->slave_address = cfg->i2c_address;

    if (  i2c_master_open( &ctx->i2c, &i2c_cfg ) != I2C_SUCCESS )
    {
        return TEMPHUM9_INIT_ERROR;
    }

    

    return TEMPHUM9_OK;

}

void temphum9_default_cfg ( temphum9_t *ctx )
{
    // Click default configuration 
}

void temphum9_generic_write ( temphum9_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len )
{
    i2c_master_transfer_t i2c_transfer;
    uint8_t tx_buf[ 256 ];
    uint8_t cnt;
    
    tx_buf[ 0 ] = reg;
    
    for ( cnt = 1; cnt <= len; cnt++ )
    {
        tx_buf[ cnt ] = data_buf[ cnt - 1 ]; 
    }
    
    i2c_transfer.slave_address = ctx->slave_address;
    i2c_transfer.buffer   = tx_buf;
    i2c_transfer.count    = len + 1;
    i2c_transfer.end_mode = I2C_STOP_MODE;

    i2c_master_start( &ctx->i2c );
    i2c_master_write( &ctx->i2c, &i2c_transfer );    
}

void temphum9_generic_read ( temphum9_t *ctx, uint8_t reg, uint8_t *data_buf, uint8_t len )
{
    i2c_master_transfer_t i2c_transfer;
    uint8_t tx_buf[ 1 ];

    tx_buf[ 0 ] = reg;
    
    i2c_transfer.slave_address = ctx->slave_address;
    i2c_transfer.buffer   = tx_buf;
    i2c_transfer.count    = 1;
    i2c_transfer.end_mode = I2C_RESTART_MODE;

    i2c_master_start( &ctx->i2c );
    i2c_master_write( &ctx->i2c, &i2c_transfer );
    
    i2c_transfer.buffer   = data_buf;
    i2c_transfer.count    = len;
    i2c_transfer.end_mode = I2C_STOP_MODE;

    i2c_master_read( &ctx->i2c, &i2c_transfer );
}

void temphum9_read_register ( temphum9_t *ctx, uint16_t register_address, uint8_t n_data, uint16_t *register_buffer )
{
    uint8_t write_buffer[ 2 ];
    uint8_t read_buffer[ 6 ];
    
    write_buffer[ 0 ] = register_address  >> 8;
    write_buffer[ 1 ] = register_address ;


    generic_read_data ( ctx, register_address, read_buffer, n_data );  

    if (n_data  ==  TEMPHUM9_SINGLE_DATA)
    {
        register_buffer[ 0 ] = (uint16_t) read_buffer[ 0 ];
        register_buffer[ 0 ] <<= 8;
        register_buffer[ 0 ] |= (uint16_t) read_buffer[ 1 ];
    }
    else if (n_data == TEMPHUM9_DUAL_DATA)
    {
        register_buffer[ 0 ] = (uint16_t) read_buffer[ 0 ];
        register_buffer[ 0 ] <<= 8;
        register_buffer[ 0 ] |= (uint16_t) read_buffer[ 1 ];
        
        register_buffer[ 1 ] = (uint16_t) read_buffer[ 3 ];
        register_buffer[ 1 ] <<= 8;
        register_buffer[ 1 ] |= (uint16_t) read_buffer[ 4] ;
    }
}

void temphum9_send_command ( temphum9_t *ctx, uint16_t command )
{
    uint8_t write_buffer[ 1 ];

    temphum9_generic_write ( ctx, command, write_buffer, 0 );
}

float temphum9_get_temperature ( temphum9_t *ctx, uint8_t data_mode )
{
    float temperature;
    uint16_t register_buffer[ 1 ];

    if ( data_mode  == TEMPHUM9_NORMAL_MODE )
    {
        temphum9_read_register ( ctx, 0x7CA2,  TEMPHUM9_SINGLE_DATA, register_buffer );
    }
    else if ( data_mode == TEMPHUM9_LOW_POWER_MODE )
    {
        temphum9_read_register ( ctx, 0x6458, TEMPHUM9_SINGLE_DATA, register_buffer );
    }

    temperature  = ( float ) register_buffer[ 0 ];
    temperature  *= 175;
    temperature  /= 65536;
    temperature  -= 45;
    
    return temperature ;
}

float temphum9_get_relative_humidity ( temphum9_t *ctx, uint8_t data_mode )
{
    float relative_humidity ;
    uint16_t register_buffer[ 1 ];

    if ( data_mode ==  TEMPHUM9_NORMAL_MODE )
    {
        temphum9_read_register ( ctx, 0x5C24, TEMPHUM9_SINGLE_DATA, register_buffer );
    }
    else if (data_mode  ==  TEMPHUM9_LOW_POWER_MODE )
    {
        temphum9_read_register ( ctx, 0x44DE,  TEMPHUM9_SINGLE_DATA, register_buffer );
    }

    relative_humidity  = ( float ) register_buffer[ 0 ]  ;
    relative_humidity  *= 100;
    relative_humidity  /= 65536;

    return relative_humidity ;
}

void temhum9_get_temperature_and_humidity ( temphum9_t *ctx,  uint8_t data_mode , float *measurement_data )
{
    uint16_t register_buffer[ 2 ];
    
    if ( data_mode  ==  TEMPHUM9_NORMAL_MODE )
    {
        temphum9_read_register ( ctx, 0x7CA2,  TEMPHUM9_DUAL_DATA, register_buffer );
    }
    else if ( data_mode  ==  TEMPHUM9_LOW_POWER_MODE )
    {
        temphum9_read_register ( ctx, 0x6458, TEMPHUM9_DUAL_DATA, register_buffer );
    }
    
    measurement_data[ 0 ] = (float) register_buffer [ 0 ];
    measurement_data[ 0 ] *= 175;
    measurement_data[ 0 ] /= 65536;
    measurement_data[ 0 ] -= 45;

    measurement_data[ 1 ] = (float) register_buffer [ 1 ];
    measurement_data[ 1 ] *= 100;
    measurement_data[ 1 ] /= 65536;
}
// ----------------------------------------------- PRIVATE FUNCTION DEFINITIONS

static void generic_read_data ( temphum9_t *ctx, uint16_t reg, uint8_t *data_buf, uint8_t len )
{
    i2c_master_transfer_t i2c_transfer;
    uint8_t tx_buf[ 2 ];

    tx_buf[ 0 ] = reg >> 8;
    tx_buf[ 1 ] = reg;
    
    i2c_transfer.slave_address = ctx->slave_address;
    i2c_transfer.buffer   = tx_buf;
    i2c_transfer.count    = 2;
    i2c_transfer.end_mode = I2C_RESTART_MODE;

    i2c_master_start( &ctx->i2c );
    i2c_master_write( &ctx->i2c, &i2c_transfer );
    
    i2c_transfer.buffer   = data_buf;
    i2c_transfer.count    = len;
    i2c_transfer.end_mode = I2C_STOP_MODE;

    i2c_master_read( &ctx->i2c, &i2c_transfer );
}

static void temphum9_write_data ( temphum9_t *ctx, uint16_t reg, uint8_t *data_buf, uint8_t len )
{
    i2c_master_transfer_t i2c_transfer;
    uint8_t tx_buf[ 256 ];
    uint8_t cnt;
    
    tx_buf[ 0 ] = reg >> 8;
    tx_buf[ 1 ] = reg; 

    for ( cnt = 2; cnt <= len; cnt++ )
    {
        tx_buf [ cnt ] = data_buf [ cnt - 1 ]; 
    } 
    
    i2c_transfer.slave_address = ctx->slave_address;
    i2c_transfer.buffer   = tx_buf;
    i2c_transfer.count    = len + 2;
    i2c_transfer.end_mode = I2C_STOP_MODE;

    i2c_master_start( &ctx->i2c );
    i2c_master_write( &ctx->i2c, &i2c_transfer );

}


// ------------------------------------------------------------------------- END

